/***************************************************************************
 *   Copyright (C) 2008 by S. MANKOWSKI / G. DE BURE skrooge@mankowski.fr  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program.  If not, see <http://www.gnu.org/licenses/>  *
 ***************************************************************************/
/** @file
* This file defines classes SKGImportKmy.
*
* @author Stephane MANKOWSKI / Guillaume DE BURE
 */
#include "skgimportkmy.h"
#include "skgtraces.h"
#include "skgservices.h"
#include "skgbankincludes.h"
#include "skgobjectbase.h"
#include "skgimportexportmanager.h"

#include <klocale.h>
#include <kfilterdev.h>

#include <QDomDocument>
#include <QFileInfo>

QSet<QString> SKGImportKmy::opTreated;
QMap<QString, SKGUnitObject> SKGImportKmy::mapIdUnit;
QMap<QString, SKGAccountObject> SKGImportKmy::mapIdAccount;
QMap<QString, SKGCategoryObject> SKGImportKmy::mapIdCategory;
QMap<QString, QString> SKGImportKmy::mapPayeeId;

SKGError SKGImportKmy::importFile(SKGImportExportManager* iImporter, SKGDocumentBank* iDocument)
{
    SKGError err;
    SKGTRACEINRC ( 2, "SKGImportKmy::importFile", err );
    QString iFileName=iImporter->getFileName();
    SKGTRACEL ( 10 ) << "Input filename=" << iFileName << endl;

    //Initialisation
    opTreated.clear();
    mapIdUnit.clear();
    mapIdAccount.clear();
    mapIdCategory.clear();
    mapPayeeId.clear();

    //Open file
    QIODevice* file=KFilterDev::deviceForFile(iFileName, "application/x-gzip");
    if ( !file->open ( QIODevice::ReadOnly ) )
    {
        err.setReturnCode ( ERR_INVALIDARG );
        err.setMessage ( i18nc ( "Error message",  "Open file '%1' failed",iFileName ) );
    }
    else
    {
        QDomDocument doc;

        //Set the file without uncompression
        QString errorMsg;
        int errorLine=0;
        int errorCol=0;
        bool contentOK=doc.setContent ( file->readAll(), &errorMsg, &errorLine, &errorCol );
        file->close();

        if ( !contentOK )
        {
            err.setReturnCode ( ERR_ABORT );
            err.setMessage ( i18nc ( "Error message",  "%1-%2: '%3'",errorLine, errorCol, errorMsg ) );

            err.addError ( ERR_INVALIDARG, i18nc ( "Error message",  "Invalid XML content in file '%1'",iFileName ) );
        }
        else
        {
            //Get root
            QDomElement docElem = doc.documentElement();
            err=iDocument->beginTransaction ( "#INTERNAL#", 6 );

            //Step 1-Get units
            QDomElement securities=docElem.firstChildElement ( "SECURITIES" );
            if ( err.isSucceeded() && !securities.isNull() )
            {
                SKGTRACEINRC ( 10, "SKGImportExportManager::importKMY-SECURITIES", err );
                QDomNodeList securityList=securities.elementsByTagName ( "SECURITY" );
                int nb=securityList.count();
                err=iDocument->beginTransaction ( "#INTERNAL#", nb );
                for ( int i=0; err.isSucceeded() && i<nb; ++i )
                {
                    QDomElement security=securityList.at ( i ).toElement();
                    QString unitName=security.attribute ( "name" );

                    //We try a creation
                    SKGUnitObject unitObj ( iDocument );
                    SKGUnitObject::createCurrencyUnit ( iDocument, unitName, unitObj );

                    if ( err.isSucceeded() && !unitObj.exist() )
                    {
                        //Creation of unit
                        err=unitObj.setName ( unitName );
                        if ( err.isSucceeded() ) err=unitObj.setSymbol ( security.attribute ( "symbol" ) );
                        if ( err.isSucceeded() ) err=unitObj.setCountry ( security.attribute ( "trading-market" ) );
                        if ( err.isSucceeded() ) err=unitObj.setType ( SKGUnitObject::SHARE );
                        if ( err.isSucceeded() )
                        {
                            //Set pairs
                            QDomNodeList pairList=security.elementsByTagName ( "PAIR" );
                            int nb2=pairList.count();
                            for ( int j=0; err.isSucceeded() && j<nb2; ++j )
                            {
                                QDomElement pair=pairList.at ( j ).toElement();
                                if ( pair.attribute ( "key" ).toLower() =="kmm-security-id" ) err=unitObj.setInternetCode ( pair.attribute ( "value" ) );
                            }
                        }
                        if ( err.isSucceeded() ) err=unitObj.save();
                    }

                    mapIdUnit[security.attribute ( "id" ) ]=unitObj;

                    if ( err.isSucceeded() ) err=iDocument->stepForward ( i+1 );
                }

                if ( err.isSucceeded() ) err=iDocument->endTransaction ( true );
                else  iDocument->endTransaction ( false );
            }
            if ( err.isSucceeded() ) err=iDocument->stepForward ( 1 );

            //Step 2-Get units prices
            QDomElement prices=docElem.firstChildElement ( "PRICES" );
            if ( err.isSucceeded() && !prices.isNull() )
            {
                SKGTRACEINRC ( 10, "SKGImportExportManager::importKMY-PRICES", err );
                QDomNodeList pricepairList=prices.elementsByTagName ( "PRICEPAIR" );
                int nb=pricepairList.count();
                err=iDocument->beginTransaction ( "#INTERNAL#", nb );
                for ( int i=0; err.isSucceeded() && i<nb; ++i )
                {
                    QDomElement pricepair=pricepairList.at ( i ).toElement();

                    SKGUnitObject unitObj=mapIdUnit[pricepair.attribute ( "from" ) ];
                    if ( unitObj.exist() )
                    {
                        //Unit is existing
                        QDomNodeList pricesList=pricepair.elementsByTagName ( "PRICE" );
                        int nb2=pricesList.count();
                        for ( int j=0; err.isSucceeded() && j<nb2; ++j )
                        {
                            QDomElement price=pricesList.at ( j ).toElement();

                            SKGUnitValueObject unitValObj;
                            err=unitObj.addUnitValue ( unitValObj );
                            if ( err.isSucceeded() ) err=unitValObj.setDate ( QDate::fromString ( price.attribute ( "date" ), "yyyy-MM-dd" ) );
                            if ( err.isSucceeded() )
                            {
                                double q=0;
                                QStringList vals=SKGServices::splitCSVLine ( price.attribute ( "price" ),'/' );
                                if ( vals.count() ==1 ) q=SKGServices::stringToDouble ( vals.at ( 0 ) );
                                else if ( vals.count() ==2 ) q=SKGServices::stringToDouble ( vals.at ( 0 ) ) /SKGServices::stringToDouble ( vals.at ( 1 ) );

                                err=unitValObj.setQuantity ( q );
                            }
                            if ( err.isSucceeded() ) err=unitValObj.save ( true, false ); //Save only without reload
                        }
                    }

                    if ( err.isSucceeded() ) err=iDocument->stepForward ( i+1 );
                }

                if ( err.isSucceeded() ) err=iDocument->endTransaction ( true );
                else  iDocument->endTransaction ( false );
            }
            if ( err.isSucceeded() ) err=iDocument->stepForward ( 2 );

            //Step 3-Create account and categories
            //Create bank and temporary account for kmymoney import
            SKGAccountObject kmymoneyTemporaryAccount ( iDocument );
            SKGBankObject bank ( iDocument );
            if ( err.isSucceeded() ) err=iDocument->addOrModifyAccount ( "KMYMONEY-TEMPORARY-ACCOUNT", "", "KMYMONEY" );
            if ( err.isSucceeded() ) err=bank.setName ( "KMYMONEY" );
            if ( err.isSucceeded() ) err=bank.load();
            if ( err.isSucceeded() ) err=kmymoneyTemporaryAccount.setName ( "KMYMONEY-TEMPORARY-ACCOUNT" );
            if ( err.isSucceeded() ) err=kmymoneyTemporaryAccount.load();

            QDomElement accounts=docElem.firstChildElement ( "ACCOUNTS" );
            if ( err.isSucceeded() && !accounts.isNull() )
            {
                SKGTRACEINRC ( 10, "SKGImportExportManager::importKMY-ACCOUNTS", err );
                QDomNodeList accountList=accounts.elementsByTagName ( "ACCOUNT" );
                int nb=accountList.count();
                err=iDocument->beginTransaction ( "#INTERNAL#", nb );
                for ( int i=0; err.isSucceeded() && i<nb; ++i )
                {
                    //Get account object
                    QDomElement account=accountList.at ( i ).toElement();
                    QString type=account.attribute ( "type" );
                    if ( type=="15" )
                    {
                        //Actions: the real account is the parent
                        mapIdAccount[account.attribute ( "id" ) ]=mapIdAccount[account.attribute ( "parentaccount" ) ];
                        mapIdUnit[account.attribute ( "id" ) ]=mapIdUnit[account.attribute ( "currency" ) ];
                    }
                    else if ( type!="12" && type!="13" && type!="16" )
                    {
                        //Creation of the bank
                        SKGBankObject bankObj ( iDocument );
                        QString bankName=account.attribute ( "institution" );
                        if ( bankName.isEmpty() ) bankObj=bank;
                        else
                        {
                            err=bankObj.setName ( bankName );
                            if ( err.isSucceeded() ) err=bankObj.save();
                        }

                        //Creation of the account
                        SKGAccountObject accountObj;
                        if ( err.isSucceeded() ) err=bankObj.addAccount ( accountObj );
                        if ( err.isSucceeded() ) err=accountObj.setName ( account.attribute ( "name" ) );
                        if ( err.isSucceeded() ) err=accountObj.setNumber ( account.attribute ( "number" ) );
                        if ( err.isSucceeded() ) err=accountObj.setComment ( account.attribute ( "description" ) );
                        if ( err.isSucceeded() )
                        {
                            QString type=account.attribute ( "type" );
                            SKGAccountObject::AccountType typeA= ( type=="4" ? SKGAccountObject::CREDITCARD : ( type=="7" ? SKGAccountObject::INVESTMENT : ( type=="9" ? SKGAccountObject::ASSETS : SKGAccountObject::CURRENT ) ) );

                            err=accountObj.setType ( typeA );
                        }
                        if ( err.isSucceeded() )
                        {
                            //Set pairs
                            QDomNodeList pairList=account.elementsByTagName ( "PAIR" );
                            int nb2=pairList.count();
                            for ( int j=0; err.isSucceeded() && j<nb2; ++j )
                            {
                                QDomElement pair=pairList.at ( j ).toElement();
                                if ( pair.attribute ( "key" ).toLower() =="mm-closed" && pair.attribute ( "value" ).toLower() =="yes" ) err=accountObj.setClosed ( true );
                                else if ( pair.attribute ( "key" ).toLower() =="preferredaccount" && pair.attribute ( "value" ).toLower() =="yes" ) err=accountObj.bookmark ( true );
                            }
                        }
                        if ( err.isSucceeded() ) err=accountObj.save();

                        mapIdAccount[account.attribute ( "id" ) ]=accountObj;
                    }
                    else if ( type!="16" )
                    {
                        //Create category
                        SKGCategoryObject cat=mapIdCategory[account.attribute ( "id" ) ];
                        if ( cat.exist() )
                        {
                            //Already existing ==> we must set the right name
                            err=cat.setName ( account.attribute ( "name" ) );
                            if ( err.isSucceeded() ) err=cat.save();
                            mapIdCategory[account.attribute ( "id" ) ]=cat;
                        }
                        else
                        {
                            //We must create it
                            cat=SKGCategoryObject ( iDocument );
                            err=cat.setName ( account.attribute ( "name" ) );
                            if ( err.isSucceeded() ) err=cat.save();
                            mapIdCategory[account.attribute ( "id" ) ]=cat;
                        }

                        //Create sub categories
                        QDomNodeList subaccountList=account.elementsByTagName ( "SUBACCOUNT" );
                        int nb2=subaccountList.count();
                        for ( int j=0; err.isSucceeded() && j<nb2; ++j )
                        {
                            QDomElement subaccount=subaccountList.at ( j ).toElement();

                            //Is child already existing ?
                            SKGCategoryObject cat2=mapIdCategory[subaccount.attribute ( "id" ) ];
                            if ( cat2.exist() )
                            {
                                //Yes ==> reparent
                                err=cat2.setParentCategory ( cat );
                                if ( err.isSucceeded() ) err=cat2.save();
                            }
                            else
                            {
                                //No ==> create
                                if ( err.isSucceeded() ) err=cat.addCategory ( cat2 );
                                if ( err.isSucceeded() ) err=cat2.setName ( subaccount.attribute ( "id" ) );
                                if ( err.isSucceeded() ) err=cat2.save();
                                mapIdCategory[subaccount.attribute ( "id" ) ]=cat2;
                            }
                        }
                    }

                    if ( err.isSucceeded() ) err=iDocument->stepForward ( i+1 );
                }

                if ( err.isSucceeded() ) err=iDocument->endTransaction ( true );
                else  iDocument->endTransaction ( false );
            }
            if ( err.isSucceeded() ) err=iDocument->stepForward ( 3 );

            //Step 4-Read payees
            QMap<QString, QString> mapIdPayee;
            QDomElement payees=docElem.firstChildElement ( "PAYEES" );
            if ( err.isSucceeded() && !accounts.isNull() )
            {
                SKGTRACEINRC ( 10, "SKGImportExportManager::importKMY-PAYEES", err );
                QDomNodeList payeeList=payees.elementsByTagName ( "PAYEE" );
                int nb=payeeList.count();
                for ( int i=0; i<nb; ++i )
                {
                    //Get account object
                    QDomElement payee=payeeList.at ( i ).toElement();
                    mapIdPayee[payee.attribute ( "id" ) ]=payee.attribute ( "name" );
                }
            }
            if ( err.isSucceeded() ) err=iDocument->stepForward ( 4 );

            //Step 5-Create operations
            if ( err.isSucceeded() )
            {
                SKGTRACEINRC ( 10, "SKGImportExportManager::importKMY-TRANSACTIONS", err );
                QDomNodeList transactionList=docElem.elementsByTagName ( "TRANSACTION" );
                int nb=transactionList.count();
                err=iDocument->beginTransaction ( "#INTERNAL#", nb );
                for ( int i=0; err.isSucceeded() && i<nb; ++i )
                {
                    //Get operation object
                    QDomElement operation=transactionList.at ( i ).toElement();

                    SKGOperationObject operationObj;
                    err=kmymoneyTemporaryAccount.addOperation ( operationObj );
                    if ( err.isSucceeded() ) err=operationObj.setDate ( QDate::fromString ( operation.attribute ( "postdate" ), "yyyy-MM-dd" ) );
                    if ( err.isSucceeded() ) err=operationObj.setComment ( operation.attribute ( "memo" ) );
                    if ( err.isSucceeded() ) err=operationObj.setImported ( true );
                    if ( err.isSucceeded() ) err=operationObj.setImportID ( "KMY-"+operation.attribute ( "id" ) );
                    if ( err.isSucceeded() )
                    {
                        QString unitName=operation.attribute ( "commodity" );
                        SKGUnitObject unit ( iDocument );
                        err=unit.setName ( unitName );
                        if ( err.isSucceeded() ) err=unit.setSymbol ( unitName );
                        if ( err.isSucceeded() )
                        {
                            if ( unit.exist() )  err=unit.load();
                            else  err=unit.save();

                            if ( err.isSucceeded() ) err=operationObj.setUnit ( unit );
                        }
                    }
                    if ( err.isSucceeded() ) err=operationObj.save();

                    //Is it a schedule ?
                    if ( err.isSucceeded() )
                    {
                        QDomElement recu=operation.parentNode().toElement();
                        if ( recu.tagName() =="SCHEDULED_TX" )
                        {
                            //Yes ==> creation of the schedule
                            if ( err.isSucceeded() ) err=operationObj.setTemplate ( true );
                            if ( err.isSucceeded() ) err=operationObj.save();

                            //Yes ==> creation of the schedule
                            SKGRecurrentOperationObject recuObj;
                            if ( err.isSucceeded() ) err=operationObj.addRecurrentOperation ( recuObj );
                            QDate firstDate=QDate::fromString ( recu.attribute ( "startDate" ), "yyyy-MM-dd" );
                            if ( err.isSucceeded() ) err=recuObj.setDate ( firstDate );
                            if ( err.isSucceeded() ) err=recuObj.autoWriteEnabled ( recu.attribute ( "autoEnter" ) =="1" );
                            if ( err.isSucceeded() ) err=recuObj.setAutoWriteDays ( 0 );

                            int occu=SKGServices::stringToInt ( recu.attribute ( "occurenceMultiplier" ) );
                            QString occuType=recu.attribute ( "occurence" ); // krazy:exclude=spelling
                            SKGRecurrentOperationObject::PeriodUnit period;
                            if ( occuType=="32" ) period=SKGRecurrentOperationObject::MONTH;
                            else if ( occuType=="1" )
                            {
                                period=SKGRecurrentOperationObject::DAY;
                                if ( err.isSucceeded() ) err=recuObj.timeLimit ( true );
                                if ( err.isSucceeded() ) err=recuObj.setTimeLimit ( 1 );
                            }
                            else if ( occuType=="2" ) period=SKGRecurrentOperationObject::DAY;
                            else if ( occuType=="4" )
                            {
                                period=SKGRecurrentOperationObject::DAY;
                                occu*=7;
                            }
                            else if ( occuType=="18" )
                            {
                                period=SKGRecurrentOperationObject::DAY;
                                occu*=15;
                            }
                            else period=SKGRecurrentOperationObject::YEAR;


                            if ( err.isSucceeded() ) err=recuObj.setPeriodIncrement ( occu );
                            if ( err.isSucceeded() ) err=recuObj.setPeriodUnit ( period );

                            if ( err.isSucceeded() )
                            {
                                QString endDate=recu.attribute ( "endDate" );
                                if ( !endDate.isEmpty() )
                                {
                                    if ( err.isSucceeded() ) err=recuObj.timeLimit ( true );
                                    if ( err.isSucceeded() ) err=recuObj.setTimeLimit ( QDate::fromString ( recu.attribute ( "endDate" ), "yyyy-MM-dd" ) );
                                }
                            }

                            if ( err.isSucceeded() ) err=recuObj.save();
                        }
                    }

                    //Get splits
                    bool parentSet=false;
                    double quantity=0;
                    QDomElement splits=operation.firstChildElement ( "SPLITS" );

                    QList<QDomNode> suboperationsList;
                    int nbSuboperations=0;
                    {
                        QDomNodeList suboperationsListTmp=splits.elementsByTagName ( "SPLIT" );
                        nbSuboperations=suboperationsListTmp.count();

                        for ( int j=0; err.isSucceeded() && j<nbSuboperations; ++j )
                        {
                            QDomElement suboperation=suboperationsListTmp.at ( j ).toElement();
                            if ( mapIdCategory.contains ( suboperation.attribute ( "account" ) ) )
                            {
                                suboperationsList.push_front ( suboperation );
                            }
                            else
                            {
                                suboperationsList.push_back ( suboperation );
                            }
                        }
                    }

                    SKGSubOperationObject suboperationObj;
                    for ( int j=0; err.isSucceeded() && j<nbSuboperations; ++j )
                    {
                        QDomElement suboperation=suboperationsList.at ( j ).toElement();

                        //Set operation attributes
                        if ( err.isSucceeded() ) err=operationObj.setPayee ( mapIdPayee[suboperation.attribute ( "payee" ) ] );
                        if ( err.isSucceeded() && operationObj.getComment().isEmpty() ) err=operationObj.setComment ( suboperation.attribute ( "memo" ) );
                        if ( err.isSucceeded() ) err=operationObj.setNumber ( SKGServices::stringToInt ( suboperation.attribute ( "number" ) ) );

                        //Set state
                        if ( operationObj.getStatus() ==SKGOperationObject::NONE )
                        {
                            QString val=suboperation.attribute ( "reconcileflag" );
                            if ( err.isSucceeded() ) err=operationObj.setStatus ( val=="1" ? SKGOperationObject::POINTED : val=="2" ? SKGOperationObject::CHECKED : SKGOperationObject::NONE );
                        }
                        if ( err.isSucceeded() ) err=operationObj.save();


                        if ( mapIdCategory.contains ( suboperation.attribute ( "account" ) ) )
                        {
                            //It is a split on category
                            SKGCategoryObject cat=mapIdCategory[suboperation.attribute ( "account" ) ];

                            double q=0;
                            QStringList vals=SKGServices::splitCSVLine ( suboperation.attribute ( "shares" ),'/' );
                            if ( vals.count() ==1 ) q=-SKGServices::stringToDouble ( vals.at ( 0 ) );
                            else if ( vals.count() ==2 ) q=-SKGServices::stringToDouble ( vals.at ( 0 ) ) /SKGServices::stringToDouble ( vals.at ( 1 ) );

                            if ( err.isSucceeded() && ( !suboperationObj.exist() || suboperationObj.getQuantity() !=q ) ) err=operationObj.addSubOperation ( suboperationObj );
                            if ( err.isSucceeded() ) err=suboperationObj.setQuantity ( q );
                            if ( err.isSucceeded() ) err=suboperationObj.setCategory ( cat );
                            if ( err.isSucceeded() ) err=suboperationObj.setComment ( suboperation.attribute ( "memo" ) );
                            if ( err.isSucceeded() ) err=suboperationObj.save();
                        }
                        else if ( !mapIdAccount.contains ( suboperation.attribute ( "account" ) ) )
                        {
                            //Set as initial balance
                            if ( err.isSucceeded() ) err=operationObj.setAttribute ( "d_date", "0000-00-00" );
                            if ( err.isSucceeded() ) err=operationObj.setStatus ( SKGOperationObject::CHECKED );
                            if ( err.isSucceeded() ) err=operationObj.save();
                        }
                        else
                        {
                            //It is a transfer of account
                            SKGAccountObject act=mapIdAccount[suboperation.attribute ( "account" ) ];

                            if ( parentSet )
                            {
                                //If the parent is already set, it means that is a transfer
                                SKGOperationObject operationObj2;
                                act.setClosed(false); //To be sure that the modification is possible. NOT SAVED
                                if ( err.isSucceeded() ) err=act.addOperation ( operationObj2 );
                                if ( err.isSucceeded() ) err=operationObj2.setTemplate ( operationObj.isTemplate() );
                                if ( err.isSucceeded() ) err=operationObj2.setDate ( operationObj.getDate() );
                                if ( err.isSucceeded() ) err=operationObj2.setNumber ( operationObj.getNumber() );
                                if ( err.isSucceeded() ) err=operationObj2.setComment ( operationObj.getComment() );
                                if ( err.isSucceeded() ) err=operationObj2.setPayee ( operationObj.getPayee() );
                                if ( err.isSucceeded() )
                                {
                                    QString val=suboperation.attribute ( "reconcileflag" );
                                    err=operationObj2.setStatus ( val=="1" ? SKGOperationObject::POINTED : val=="2" ? SKGOperationObject::CHECKED : SKGOperationObject::NONE );
                                }
                                if ( err.isSucceeded() ) err=operationObj2.setImported ( true );
                                if ( err.isSucceeded() ) err=operationObj2.setImportID ( operationObj.getImportID() );

                                if (mapIdUnit.contains(suboperation.attribute ( "account" )))
                                {
                                    if ( err.isSucceeded() ) err=operationObj2.setUnit ( mapIdUnit[suboperation.attribute ( "account" )] );
                                }
                                else
                                {
                                    SKGUnitObject unit;
                                    if ( err.isSucceeded() ) err=operationObj.getUnit ( unit );
                                    if ( err.isSucceeded() ) err=operationObj2.setUnit ( unit );
                                }
                                if ( err.isSucceeded() ) err=operationObj2.save();
                                if ( err.isSucceeded() ) err=operationObj2.setGroupOperation ( operationObj );
                                if ( err.isSucceeded() ) err=operationObj2.save();

                                //Create sub operation on operationObj2
                                SKGSubOperationObject suboperationObj2;
                                if ( err.isSucceeded() ) err=operationObj2.addSubOperation ( suboperationObj2 );
                                if ( err.isSucceeded() )
                                {
                                    //We must take the quality of the split having an action
                                    QStringList vals=SKGServices::splitCSVLine ( suboperation.attribute ( "shares" ),'/' );
                                    if ( vals.count() ==1 ) quantity=SKGServices::stringToDouble ( vals.at ( 0 ) );
                                    else if ( vals.count() ==2 ) quantity=SKGServices::stringToDouble ( vals.at ( 0 ) ) /SKGServices::stringToDouble ( vals.at ( 1 ) );
                                    err=suboperationObj2.setQuantity ( quantity );
                                }
                                if ( err.isSucceeded() ) err=suboperationObj2.setComment ( suboperation.attribute ( "memo" ) );
                                if ( err.isSucceeded() ) err=suboperationObj2.save();
                            }
                            else
                            {
                                //We set the parent
                                act.setClosed(false); //To be sure that the modification is possible. NOT SAVED
                                if ( err.isSucceeded() ) err=operationObj.setParentAccount ( act );
                                if ( err.isSucceeded() ) err=operationObj.save();

                                //Compute quantity
                                QStringList vals=SKGServices::splitCSVLine ( suboperation.attribute ( "shares" ),'/' );
                                if ( vals.count() ==1 ) quantity=SKGServices::stringToDouble ( vals.at ( 0 ) );
                                else if ( vals.count() ==2 ) quantity=SKGServices::stringToDouble ( vals.at ( 0 ) ) /SKGServices::stringToDouble ( vals.at ( 1 ) );

                                //Create sub operation on operationObj
                                quantity-=SKGServices::stringToDouble ( operationObj.getAttribute ( "f_QUANTITY" ) );
                                if ( quantity!=0 )
                                {
                                    if ( err.isSucceeded() ) err=operationObj.addSubOperation ( suboperationObj );
                                    if ( err.isSucceeded() ) err=suboperationObj.setQuantity ( quantity );
                                    if ( err.isSucceeded() ) err=suboperationObj.setComment ( suboperation.attribute ( "memo" ) );
                                    if ( err.isSucceeded() ) err=suboperationObj.save();
                                }

                                parentSet=true;
                            }
                        }
                    }

                    if ( err.isSucceeded() && i%100==0 ) err = SKGServices::executeSqliteOrder ( iDocument, "ANALYZE" );

                    if ( err.isSucceeded() ) err=iDocument->stepForward ( i+1 );
                }

                if ( err.isSucceeded() ) err=iDocument->endTransaction ( true );
                else  iDocument->endTransaction ( false );
            }
            if ( err.isSucceeded() ) err=iDocument->stepForward ( 5 );

            //Step 6-Remove useless account and temporary account
            {
                if ( err.isSucceeded() ) err=kmymoneyTemporaryAccount.remove ( false );
                if ( err.isSucceeded() ) err=SKGServices::executeSqliteOrder ( iDocument, "DELETE FROM account WHERE rd_bank_id="+SKGServices::intToString ( bank.getID() ) +" AND (SELECT COUNT(1) FROM operation WHERE operation.rd_account_id=account.id)=0" );
            }
            if ( err.isSucceeded() ) err=iDocument->stepForward ( 6 );

            if ( err.isSucceeded() ) err=iDocument->endTransaction ( true );
            else  iDocument->endTransaction ( false );

            if ( err.isSucceeded() ) err = SKGServices::executeSqliteOrder ( iDocument, "ANALYZE" );
        }
    }

    delete file;
    return err;
}

SKGError SKGImportKmy::exportFile(SKGImportExportManager* iImporter, SKGDocumentBank* iDocument)
{
    SKGError err;
    SKGTRACEINRC ( 2, "SKGImportKmy::exportFile", err );
    QString iFileName=iImporter->getFileName();
    SKGTRACEL ( 10 ) << "Input filename=" << iFileName << endl;
    //Open file
    QIODevice* file=KFilterDev::deviceForFile(iFileName, "application/x-gzip");
    if ( !file->open ( QIODevice::WriteOnly ) )
    {
        err.setReturnCode ( ERR_INVALIDARG );
        err.setMessage ( i18nc ( "Error message",  "Save file '%1' failed",iFileName ) );
    }
    else
    {
        QDomDocument doc ( "KMYMONEY-FILE" );
        QDomComment comment=doc.createComment ( "Generated by libskgbankmodeler" );
        doc.appendChild ( comment );

        QDomElement root = doc.createElement ( "KMYMONEY-FILE" );
        doc.appendChild ( root );
        {
            err=iDocument->beginTransaction ( "#INTERNAL#", 7 );
            if ( err.isSucceeded() )
            {
                //Step 1-<FILEINFO>
                QDomElement fileindo=doc.createElement ( "FILEINFO" );
                root.appendChild ( fileindo );

                {
                    //<CREATION_DATE>
                    QDomElement creation_date=doc.createElement ( "CREATION_DATE" );
                    fileindo.appendChild ( creation_date );
                    creation_date.setAttribute ( "date", SKGServices::dateToSqlString ( QDateTime::currentDateTime() ) );

                    //<LAST_MODIFIED_DATE>
                    QDomElement last_modified_date=doc.createElement ( "LAST_MODIFIED_DATE" );
                    fileindo.appendChild ( last_modified_date );
                    last_modified_date.setAttribute ( "date", SKGServices::dateToSqlString ( QDateTime::currentDateTime() ) );

                    //<VERSION>
                    QDomElement version=doc.createElement ( "VERSION" );
                    fileindo.appendChild ( version );
                    version.setAttribute ( "id", "1" );

                    //<FIXVERSION>
                    QDomElement fixversion=doc.createElement ( "FIXVERSION" );
                    fileindo.appendChild ( fixversion );
                    fixversion.setAttribute ( "id", "2" );
                }

                //<USER>
                QDomElement user=doc.createElement ( "USER" );
                root.appendChild ( user );
                user.setAttribute ( "email", "" );
                user.setAttribute ( "name", "" );
                {
                    //ADDRESS
                    QDomElement address=doc.createElement ( "ADDRESS" );
                    user.appendChild ( address );
                    address.setAttribute ( "street", "" );
                    address.setAttribute ( "zipcode", "" );
                    address.setAttribute ( "county", "" );
                    address.setAttribute ( "city", "" );
                    address.setAttribute ( "telephone", "" );
                }
                if ( err.isSucceeded() ) err=iDocument->stepForward ( 1 );

                ////Step 2-<INSTITUTIONS>
                QDomElement institutions=doc.createElement ( "INSTITUTIONS" );
                root.appendChild ( institutions );

                SKGObjectBase::SKGListSKGObjectBase objects;
                if ( err.isSucceeded() ) err=SKGObjectBase::getObjects ( iDocument, "bank", "(select count(1) from account where account.rd_bank_id=bank.id)>0", objects );
                int nb=objects.count();
                institutions.setAttribute ( "count", SKGServices::intToString ( nb ) );
                if ( err.isSucceeded() )
                {
                    err=iDocument->beginTransaction ( "#INTERNAL#", nb );
                    for ( int i=0; err.isSucceeded() && i<nb; ++i )
                    {
                        SKGBankObject obj=objects.at ( i );
                        QDomElement institution=doc.createElement ( "INSTITUTION" );
                        institutions.appendChild ( institution );

                        institution.setAttribute ( "id", obj.getUniqueID() );
                        institution.setAttribute ( "name", obj.getName() );
                        institution.setAttribute ( "sortcode", "" );
                        institution.setAttribute ( "manager", "" );

                        QDomElement address=doc.createElement ( "ADDRESS" );
                        institution.appendChild ( address );

                        address.setAttribute ( "street", "" );
                        address.setAttribute ( "zip", "" );
                        address.setAttribute ( "city", "" );
                        address.setAttribute ( "telephone", "" );

                        QDomElement accountids=doc.createElement ( "ACCOUNTIDS" );
                        institution.appendChild ( accountids );

                        SKGObjectBase::SKGListSKGObjectBase accounts;
                        err=obj.getAccounts ( accounts );
                        int nb2=accounts.count();
                        for ( int j=0; err.isSucceeded() && j<nb2; ++j )
                        {
                            QDomElement accountid=doc.createElement ( "ACCOUNTID" );
                            accountids.appendChild ( accountid );

                            accountid.setAttribute ( "id", accounts.at ( j ).getUniqueID() );
                        }
                        if ( err.isSucceeded() ) err=iDocument->stepForward ( i+1 );
                    }

                    if ( err.isSucceeded() ) err=iDocument->endTransaction ( true );
                    else  iDocument->endTransaction ( false );
                }
                if ( err.isSucceeded() ) err=iDocument->stepForward ( 2 );

                //Step 3-<PAYEES>
                QDomElement payees=doc.createElement ( "PAYEES" );
                root.appendChild ( payees );

                QStringList listPayees;
                if ( err.isSucceeded() ) err=SKGServices::getDistinctValues ( iDocument, "operation", "t_payee", "", listPayees );
                nb=listPayees.count();
                payees.setAttribute ( "count", SKGServices::intToString ( nb ) );
                if ( err.isSucceeded() )
                {
                    err=iDocument->beginTransaction ( "#INTERNAL#", nb );
                    for ( int i=0; err.isSucceeded() && i<nb; ++i )
                    {
                        QDomElement payee=doc.createElement ( "PAYEE" );
                        payees.appendChild ( payee );

                        payee.setAttribute ( "id", SKGServices::intToString ( i+1 ) );
                        payee.setAttribute ( "name", listPayees.at ( i ) );
                        payee.setAttribute ( "email", "" );
                        payee.setAttribute ( "matchingenabled", "0" );
                        payee.setAttribute ( "reference", "" );

                        QDomElement address=doc.createElement ( "ADDRESS" );
                        payee.appendChild ( address );

                        address.setAttribute ( "street", "" );
                        address.setAttribute ( "zip", "" );
                        address.setAttribute ( "city", "" );
                        address.setAttribute ( "telephone", "" );

                        mapPayeeId[listPayees.at ( i ) ]=SKGServices::intToString ( i+1 );

                        if ( err.isSucceeded() ) err=iDocument->stepForward ( i+1 );
                    }

                    if ( err.isSucceeded() ) err=iDocument->endTransaction ( true );
                    else  iDocument->endTransaction ( false );
                }
                if ( err.isSucceeded() ) err=iDocument->stepForward ( 3 );

                //Step 4-<ACCOUNTS>
                QDomElement accounts=doc.createElement ( "ACCOUNTS" );
                root.appendChild ( accounts );

                //Std accounts
                QString stdUnit=iDocument->getPrimaryUnit().Name;
                stdUnit = SKGUnitObject::getInternationalCode(stdUnit);
                if (stdUnit.isEmpty()) stdUnit="EUR";

                QDomElement accountEquity;
                QDomElement accountAsset;
                QDomElement accountLiability;
                QDomElement accountIncome;
                QDomElement accountExpense;
                {
                    QDomElement account=doc.createElement ( "ACCOUNT" );
                    accounts.appendChild ( account );

                    account.setAttribute ( "id", "AStd::Equity" );
                    account.setAttribute ( "name", "Equity" );
                    account.setAttribute ( "number", "" );
                    account.setAttribute ( "type", "16" );
                    account.setAttribute ( "institution", "" );
                    account.setAttribute ( "parentaccount", "" );
                    account.setAttribute ( "lastmodified", "" );
                    account.setAttribute ( "lastreconciled", "" );
                    account.setAttribute ( "opened", "" );
                    account.setAttribute ( "currency", stdUnit );
                    account.setAttribute ( "description", "" );

                    QDomElement subaccounts=doc.createElement ( "SUBACCOUNTS" );
                    account.appendChild ( subaccounts );
                    accountEquity=subaccounts;
                }

                {
                    QDomElement account=doc.createElement ( "ACCOUNT" );
                    accounts.appendChild ( account );

                    account.setAttribute ( "id", "AStd::Asset" );
                    account.setAttribute ( "name", "Asset" );
                    account.setAttribute ( "number", "" );
                    account.setAttribute ( "type", "9" );
                    account.setAttribute ( "institution", "" );
                    account.setAttribute ( "parentaccount", "" );
                    account.setAttribute ( "lastmodified", "" );
                    account.setAttribute ( "lastreconciled", "" );
                    account.setAttribute ( "opened", "" );
                    account.setAttribute ( "currency", stdUnit );
                    account.setAttribute ( "description", "" );

                    QDomElement subaccounts=doc.createElement ( "SUBACCOUNTS" );
                    account.appendChild ( subaccounts );
                    accountAsset=subaccounts;
                }

                {
                    QDomElement account=doc.createElement ( "ACCOUNT" );
                    accounts.appendChild ( account );

                    account.setAttribute ( "id", "AStd::Liability" );
                    account.setAttribute ( "name", "Liability" );
                    account.setAttribute ( "number", "" );
                    account.setAttribute ( "type", "10" );
                    account.setAttribute ( "institution", "" );
                    account.setAttribute ( "parentaccount", "" );
                    account.setAttribute ( "lastmodified", "" );
                    account.setAttribute ( "lastreconciled", "" );
                    account.setAttribute ( "opened", "" );
                    account.setAttribute ( "currency", stdUnit );
                    account.setAttribute ( "description", "" );

                    QDomElement subaccounts=doc.createElement ( "SUBACCOUNTS" );
                    account.appendChild ( subaccounts );
                    accountLiability=subaccounts;
                }

                {
                    QDomElement account=doc.createElement ( "ACCOUNT" );
                    accounts.appendChild ( account );

                    account.setAttribute ( "id", "AStd::Income" );
                    account.setAttribute ( "name", "Income" );
                    account.setAttribute ( "number", "" );
                    account.setAttribute ( "type", "12" );
                    account.setAttribute ( "institution", "" );
                    account.setAttribute ( "parentaccount", "" );
                    account.setAttribute ( "lastmodified", "" );
                    account.setAttribute ( "lastreconciled", "" );
                    account.setAttribute ( "opened", "" );
                    account.setAttribute ( "currency", stdUnit );
                    account.setAttribute ( "description", "" );

                    QDomElement subaccounts=doc.createElement ( "SUBACCOUNTS" );
                    account.appendChild ( subaccounts );
                    accountIncome=subaccounts;
                }

                {
                    QDomElement account=doc.createElement ( "ACCOUNT" );
                    accounts.appendChild ( account );

                    account.setAttribute ( "id", "AStd::Expense" );
                    account.setAttribute ( "name", "Expense" );
                    account.setAttribute ( "number", "" );
                    account.setAttribute ( "type", "13" );
                    account.setAttribute ( "institution", "" );
                    account.setAttribute ( "parentaccount", "" );
                    account.setAttribute ( "lastmodified", "" );
                    account.setAttribute ( "lastreconciled", "" );
                    account.setAttribute ( "opened", "" );
                    account.setAttribute ( "currency", stdUnit );
                    account.setAttribute ( "description", "" );

                    QDomElement subaccounts=doc.createElement ( "SUBACCOUNTS" );
                    account.appendChild ( subaccounts );
                    accountExpense=subaccounts;
                }

                if ( err.isSucceeded() ) err=SKGObjectBase::getObjects ( iDocument, "account", "", objects );
                nb=objects.count();
                if ( err.isSucceeded() )
                {
                    err=iDocument->beginTransaction ( "#INTERNAL#", nb );
                    for ( int i=0; err.isSucceeded() && i<nb; ++i )
                    {
                        SKGAccountObject obj=objects.at ( i );
                        QDomElement account=doc.createElement ( "ACCOUNT" );
                        accounts.appendChild ( account );

                        account.setAttribute ( "id", obj.getUniqueID() );
                        account.setAttribute ( "name", obj.getName() );
                        account.setAttribute ( "number", obj.getNumber() );
                        account.setAttribute ( "type", obj.getType() ==SKGAccountObject::CREDITCARD ? "4" : ( obj.getType() ==SKGAccountObject::INVESTMENT ? "7" : ( obj.getType() ==SKGAccountObject::ASSETS ? "9" : "1" ) ) );

                        SKGBankObject bank;
                        err=obj.getBank ( bank );
                        account.setAttribute ( "institution", bank.getUniqueID() );

                        account.setAttribute ( "parentaccount", "AStd::Asset" );
                        account.setAttribute ( "lastmodified", "" );
                        account.setAttribute ( "lastreconciled", "" );
                        account.setAttribute ( "opened", "" );
                        SKGUnitObject unit;
                        obj.getUnit(unit);
                        QString unitS=SKGUnitObject::getInternationalCode(unit.getName());
                        if (unitS.isEmpty()) unitS="EUR";
                        account.setAttribute ( "currency", unitS );
                        account.setAttribute ( "description", "" );

                        //Bookmarked account
                        QDomElement keyvaluepairs=doc.createElement ( "KEYVALUEPAIRS" );
                        account.appendChild ( keyvaluepairs );
                        if (obj.isBookmarked())
                        {
                            QDomElement pair=doc.createElement ( "PAIR" );
                            keyvaluepairs.appendChild ( pair );
                            pair.setAttribute ( "key", "PreferredAccount" );
                            pair.setAttribute ( "value", "Yes" );
                        }
                        //Closed account
                        if (obj.isClosed())
                        {
                            QDomElement pair=doc.createElement ( "PAIR" );
                            keyvaluepairs.appendChild ( pair );
                            pair.setAttribute ( "key", "mm-closed" );
                            pair.setAttribute ( "value", "yes" );
                        }

                        //Add it in asset
                        QDomElement subaccount=doc.createElement ( "SUBACCOUNT" );
                        accountAsset.appendChild ( subaccount );
                        subaccount.setAttribute ( "id", obj.getUniqueID() );

                        if ( err.isSucceeded() ) err=iDocument->stepForward ( i+1 );
                    }

                    if ( err.isSucceeded() ) err=iDocument->endTransaction ( true );
                    else  iDocument->endTransaction ( false );
                }
                if ( err.isSucceeded() ) err=iDocument->stepForward ( 4 );

                if ( err.isSucceeded() ) err=SKGObjectBase::getObjects ( iDocument, "category", "", objects );
                accounts.setAttribute ( "count", SKGServices::intToString ( 5+nb+objects.count() ) );
                nb=objects.count();
                if ( err.isSucceeded() )
                {
                    err=iDocument->beginTransaction ( "#INTERNAL#", nb );
                    for ( int i=0; err.isSucceeded() && i<nb; ++i )
                    {
                        SKGCategoryObject obj=objects.at ( i );
                        QDomElement account=doc.createElement ( "ACCOUNT" );
                        accounts.appendChild ( account );

                        account.setAttribute ( "id", obj.getUniqueID() );
                        account.setAttribute ( "name", obj.getName() );
                        account.setAttribute ( "number", "" );
                        account.setAttribute ( "type", obj.getCurrentAmount() <0 ? "13" : "12" );

                        account.setAttribute ( "institution", "" );

                        SKGCategoryObject parent;
                        obj.getParentCategory ( parent );

                        QString parentId= ( parent.exist() ? parent.getUniqueID() : ( obj.getCurrentAmount() <0 ? "AStd::Expense" : "AStd::Income" ) );
                        if ( parentId=="AStd::Expense" )
                        {
                            QDomElement subaccount=doc.createElement ( "SUBACCOUNT" );
                            accountExpense.appendChild ( subaccount );
                            subaccount.setAttribute ( "id", obj.getUniqueID() );
                        }
                        else if ( parentId=="AStd::Income" )
                        {
                            QDomElement subaccount=doc.createElement ( "SUBACCOUNT" );
                            accountIncome.appendChild ( subaccount );
                            subaccount.setAttribute ( "id", obj.getUniqueID() );
                        }

                        account.setAttribute ( "parentaccount", parentId );
                        account.setAttribute ( "lastmodified", "" );
                        account.setAttribute ( "lastreconciled", "" );
                        account.setAttribute ( "opened", "" );
                        account.setAttribute ( "currency", stdUnit );
                        account.setAttribute ( "description", "" );

                        QDomElement subaccounts=doc.createElement ( "SUBACCOUNTS" );
                        account.appendChild ( subaccounts );

                        SKGObjectBase::SKGListSKGObjectBase categories;
                        if ( err.isSucceeded() ) err=obj.getCategories ( categories );
                        int nb2=categories.count();
                        for ( int j=0; err.isSucceeded() && j<nb2; ++j )
                        {
                            QDomElement subaccount=doc.createElement ( "SUBACCOUNT" );
                            subaccounts.appendChild ( subaccount );

                            subaccount.setAttribute ( "id", categories.at ( j ).getUniqueID() );
                        }
                        if ( err.isSucceeded() ) err=iDocument->stepForward ( i+1 );
                    }

                    if ( err.isSucceeded() ) err=iDocument->endTransaction ( true );
                    else  iDocument->endTransaction ( false );
                }
                if ( err.isSucceeded() ) err=iDocument->stepForward ( 5 );

                //Step 5-<TRANSACTIONS>
                QDomElement transactions=doc.createElement ( "TRANSACTIONS" );
                root.appendChild ( transactions );

                if ( err.isSucceeded() ) err=SKGObjectBase::getObjects ( iDocument, "v_operation", "t_template='N'", objects );
                nb=objects.count();
                transactions.setAttribute ( "count", SKGServices::intToString ( nb ) );
                if ( err.isSucceeded() )
                {
                    err=iDocument->beginTransaction ( "#INTERNAL#", nb );
                    for ( int i=0; err.isSucceeded() && i<nb; ++i )
                    {
                        err=exportOperation(objects.at ( i ), doc, transactions);
                        if ( err.isSucceeded() ) err=iDocument->stepForward ( i+1 );
                    }

                    if ( err.isSucceeded() ) err=iDocument->endTransaction ( true );
                    else  iDocument->endTransaction ( false );
                }

                //<KEYVALUEPAIRS>
                QDomElement keyvaluepairs=doc.createElement ( "KEYVALUEPAIRS" );
                root.appendChild ( keyvaluepairs );
                {
                    QDomElement pair=doc.createElement ( "PAIR" );
                    keyvaluepairs.appendChild ( pair );
                    pair.setAttribute ( "key", "kmm-baseCurrency" );
                    pair.setAttribute ( "value", stdUnit );
                }

                //<SCHEDULES>
                QDomElement schedules=doc.createElement ( "SCHEDULES" );
                root.appendChild ( schedules );

                if ( err.isSucceeded() ) err=SKGObjectBase::getObjects ( iDocument, "recurrentoperation", "", objects );
                nb=objects.count();
                transactions.setAttribute ( "count", SKGServices::intToString ( nb ) );
                if ( err.isSucceeded() )
                {
                    err=iDocument->beginTransaction ( "#INTERNAL#", nb );
                    for ( int i=0; err.isSucceeded() && i<nb; ++i )
                    {
                        SKGRecurrentOperationObject obj=objects.at(i);
                        SKGOperationObject op;
                        err=obj.getParentOperation(op);
                        if (err.isSucceeded())
                        {
                            QDomElement scheduled_tx=doc.createElement ( "SCHEDULED_TX" );
                            schedules.appendChild ( scheduled_tx );

                            scheduled_tx.setAttribute ( "id", obj.getUniqueID() );
                            scheduled_tx.setAttribute ( "name", obj.getUniqueID() );
                            scheduled_tx.setAttribute ( "startDate", obj.getAttribute ( "d_date" ));
                            scheduled_tx.setAttribute ( "lastPayment", obj.getAttribute ( "d_date" ));
                            bool autoEnter=false;
                            obj.autoWriteEnabled (autoEnter);
                            scheduled_tx.setAttribute ( "autoEnter", autoEnter  ? "1" : "0");

                            QString occuType;
                            int occu=obj.getPeriodIncrement();
                            SKGRecurrentOperationObject::PeriodUnit punit=obj.getPeriodUnit();
                            if (punit==SKGRecurrentOperationObject::MONTH) occuType="32";
                            else if (punit==SKGRecurrentOperationObject::DAY) occuType='2';
                            else occuType="16384";

                            scheduled_tx.setAttribute ( "occurenceMultiplier", SKGServices::intToString(occu));
                            scheduled_tx.setAttribute ( "occurence", occuType);  // krazy:exclude=spelling
                            scheduled_tx.setAttribute ( "weekendOption", "0");
                            scheduled_tx.setAttribute ( "paymentType", "1");
                            QChar type='1';
                            SKGOperationObject op2;
                            if (op.isTransfer(op)) type='4';
                            else if (op.getCurrentAmount()>0) type='2';
                            scheduled_tx.setAttribute ( "type", type);
                            scheduled_tx.setAttribute ( "fixed", "1");

                            QString endDate;
                            if (obj.hasTimeLimit())
                            {
                                QDate firstDate=obj.getDate();

                                //We must compute the date
                                int p= occu*( obj.getTimeLimit()-1 );
                                if ( punit==SKGRecurrentOperationObject::DAY ) firstDate=firstDate.addDays ( p );
                                else if ( punit==SKGRecurrentOperationObject::MONTH ) firstDate=firstDate.addMonths ( p );
                                else if ( punit==SKGRecurrentOperationObject::YEAR ) firstDate=firstDate.addYears ( p );

                                endDate=firstDate.toString("yyyy-MM-dd");
                            }
                            scheduled_tx.setAttribute ( "endDate", endDate);

                            err=exportOperation(op, doc, scheduled_tx);
                        }
                        if ( err.isSucceeded() ) err=iDocument->stepForward ( i+1 );
                    }

                    if ( err.isSucceeded() ) err=iDocument->endTransaction ( true );
                    else  iDocument->endTransaction ( false );
                }
                if ( err.isSucceeded() ) err=iDocument->stepForward ( 6 );

                //Step 6-<SECURITIES>
                QDomElement securities=doc.createElement ( "SECURITIES" );
                root.appendChild ( securities );

                if ( err.isSucceeded() ) err=SKGObjectBase::getObjects ( iDocument, "unit", "t_type='S'", objects );
                nb=objects.count();
                securities.setAttribute ( "count", SKGServices::intToString ( nb ) );
                transactions.setAttribute ( "count", SKGServices::intToString ( nb ) );
                if ( err.isSucceeded() )
                {
                    err=iDocument->beginTransaction ( "#INTERNAL#", nb );
                    for ( int i=0; err.isSucceeded() && i<nb; ++i )
                    {
                        SKGUnitObject obj=objects.at ( i );
                        QDomElement security=doc.createElement ( "SECURITY" );
                        securities.appendChild ( security );

                        SKGUnitObject parent;
                        obj.getUnit(parent);
                        QString unitP=SKGUnitObject::getInternationalCode(parent.getName());
                        if (unitP.isEmpty()) unitP=stdUnit;

                        security.setAttribute ( "id", obj.getName() );
                        security.setAttribute ( "trading-currency", unitP );
                        security.setAttribute ( "saf", "100000" );
                        security.setAttribute ( "symbol", obj.getSymbol() );
                        security.setAttribute ( "trading-market", obj.getCountry() );
                        security.setAttribute ( "type", "0" );
                        security.setAttribute ( "name", obj.getName() );

                        QString internetCode=obj.getInternetCode();
                        if ( !internetCode.isEmpty() )
                        {
                            QDomElement keyvaluepairs=doc.createElement ( "KEYVALUEPAIRS" );
                            security.appendChild ( keyvaluepairs );

                            QDomElement pair1=doc.createElement ( "PAIR" );
                            keyvaluepairs.appendChild ( pair1 );
                            pair1.setAttribute ( "key", "kmm-online-source" );
                            pair1.setAttribute ( "value", "Yahoo" );

                            QDomElement pair2=doc.createElement ( "PAIR" );
                            keyvaluepairs.appendChild ( pair2 );
                            pair2.setAttribute ( "key", "kmm-security-id" );
                            pair2.setAttribute ( "value", internetCode );
                        }
                        if ( err.isSucceeded() ) err=iDocument->stepForward ( i+1 );
                    }

                    if ( err.isSucceeded() ) err=iDocument->endTransaction ( true );
                    else  iDocument->endTransaction ( false );
                }

                //<CURRENCIES>
                QDomElement currencies=doc.createElement ( "CURRENCIES" );
                root.appendChild ( currencies );
                QStringList units=SKGUnitObject::getListofKnownCurrencies(false);
                nb=units.count();
                int nbreal=0;
                for ( int i=0; i<nb; ++i )
                {
                    SKGServices::SKGUnitInfo info=SKGUnitObject::getUnitInfo ( units.at(i) );
                    if ( info.Name!=info.Symbol )
                    {
                        QDomElement currency=doc.createElement ( "CURRENCY" );
                        currencies.appendChild ( currency );
                        currency.setAttribute ( "saf", "100" );
                        currency.setAttribute ( "symbol", info.Symbol );
                        currency.setAttribute ( "type", "3" );
                        currency.setAttribute ( "id", SKGUnitObject::getInternationalCode(info.Name) );
                        currency.setAttribute ( "name", info.Name );
                        currency.setAttribute ( "ppu", "100" );
                        currency.setAttribute ( "scf", "100" );

                        ++nbreal;
                    }
                }
                currencies.setAttribute ( "count", SKGServices::intToString ( nbreal ) );

                //<PRICES>
                QDomElement prices=doc.createElement ( "PRICES" );
                root.appendChild ( prices );
                prices.setAttribute ( "count", "0" );

                //<REPORTS>
                QDomElement reports=doc.createElement ( "REPORTS" );
                root.appendChild ( reports );
                reports.setAttribute ( "count", "0" );

                //<BUDGETS>
                QDomElement budgets=doc.createElement ( "BUDGETS" );
                root.appendChild ( budgets );
                budgets.setAttribute ( "count", "0" );

                //Save file
                if ( err.isSucceeded() ) file->write(doc.toString().toUtf8());
                if ( err.isSucceeded() ) err=iDocument->stepForward ( 7 );
            }

            if ( err.isSucceeded() ) err=iDocument->endTransaction ( true );
            else  iDocument->endTransaction ( false );
        }

        file->close();
    }

    delete file;

    return err;
}

SKGError SKGImportKmy::exportOperation ( const SKGOperationObject& iOperation, QDomDocument& iDoc, QDomElement& iTransaction)
{
    SKGError err;
    SKGTRACEINRC ( 2, "SKGImportKmy::exportOperation", err );
    if (!opTreated.contains(iOperation.getUniqueID()))
    {
        QDomElement transaction=iDoc.createElement ( "TRANSACTION" );
        iTransaction.appendChild ( transaction );

        SKGUnitObject unit;
        iOperation.getUnit ( unit );

        QString date=iOperation.getAttribute ( "d_date" );
        transaction.setAttribute ( "id", iOperation.getUniqueID() );
        transaction.setAttribute ( "entrydate", date );
        transaction.setAttribute ( "postdate", date );
        transaction.setAttribute ( "memo", iOperation.getComment() );
        transaction.setAttribute ( "commodity", SKGUnitObject::getInternationalCode(unit.getName()) );

        QString reconcileflag= ( iOperation.getStatus() ==SKGOperationObject::POINTED ? "1" : ( iOperation.getStatus() ==SKGOperationObject::CHECKED ? "2" : "0" ) );

        SKGAccountObject act;
        if ( err.isSucceeded() ) err=iOperation.getParentAccount ( act );

        QDomElement splits=iDoc.createElement ( "SPLITS" );
        transaction.appendChild ( splits );

        QDomElement split=iDoc.createElement ( "SPLIT" );
        splits.appendChild ( split );

        //Split for account
        split.setAttribute ( "payee", mapPayeeId[iOperation.getPayee() ] );
        split.setAttribute ( "reconciledate", "" );
        split.setAttribute ( "id", iOperation.getUniqueID() );
        QString shape=SKGServices::doubleToString ( 100*SKGServices::stringToDouble ( iOperation.getAttribute ( "f_QUANTITY" ) ) ) +"/100";
        split.setAttribute ( "shares", shape );
        split.setAttribute ( "action", "" );
        split.setAttribute ( "bankid", "" );
        split.setAttribute ( "number", SKGServices::intToString ( iOperation.getNumber() ) );
        split.setAttribute ( "reconcileflag", reconcileflag );
        split.setAttribute ( "memo", iOperation.getComment() );
        QString originalAmount=iOperation.getProperty ( "SKG_OP_ORIGINAL_AMOUNT" );
        if (!originalAmount.isEmpty()) shape=SKGServices::doubleToString ( 100*SKGServices::stringToDouble ( originalAmount ) ) +"/100";
        split.setAttribute ( "value", shape );
        split.setAttribute ( "account", act.getUniqueID() );

        SKGOperationObject obj2;
        if ( err.isSucceeded() && iOperation.isTransfer ( obj2 ))
        {
            //It is a transfert
            QString reconcileflag= ( obj2.getStatus() ==SKGOperationObject::POINTED ? "1" : ( obj2.getStatus() ==SKGOperationObject::CHECKED ? "2" : "0" ) );

            SKGAccountObject act2;
            if ( err.isSucceeded() ) err=obj2.getParentAccount ( act2 );

            QDomElement split=iDoc.createElement ( "SPLIT" );
            splits.appendChild ( split );

            //Split for account
            split.setAttribute ( "payee", mapPayeeId[obj2.getPayee() ] );
            split.setAttribute ( "reconciledate", "" );
            split.setAttribute ( "id", obj2.getUniqueID() );
            QString shape=SKGServices::doubleToString ( 100*SKGServices::stringToDouble ( obj2.getAttribute ( "f_QUANTITY" ) ) ) +"/100";
            split.setAttribute ( "shares", shape );
            split.setAttribute ( "action", "" );
            split.setAttribute ( "bankid", "" );
            split.setAttribute ( "number", SKGServices::intToString ( obj2.getNumber() ) );
            split.setAttribute ( "reconcileflag", reconcileflag );
            split.setAttribute ( "memo", obj2.getComment() );
            split.setAttribute ( "value", shape );
            split.setAttribute ( "account", act2.getUniqueID() );

            opTreated.insert(obj2.getUniqueID());
        }
        else
        {
            SKGObjectBase::SKGListSKGObjectBase subops;
            if ( err.isSucceeded() ) err=iOperation.getSubOperations ( subops );
            int nb2=subops.count();
            for ( int j=0; err.isSucceeded() && j<nb2; ++j )
            {
                QDomElement split=iDoc.createElement ( "SPLIT" );
                splits.appendChild ( split );

                SKGSubOperationObject subop=subops.at ( j );
                SKGCategoryObject cat;
                subop.getCategory ( cat );
                split.setAttribute ( "payee", mapPayeeId[iOperation.getPayee() ] );
                split.setAttribute ( "reconciledate", "" );
                split.setAttribute ( "id", subop.getUniqueID() );
                QString shape=SKGServices::doubleToString ( -100*subop.getQuantity() ) +"/100";
                split.setAttribute ( "shares", shape );
                split.setAttribute ( "action", "" );
                split.setAttribute ( "bankid", "" );
                split.setAttribute ( "number", SKGServices::intToString ( iOperation.getNumber() ) );
                split.setAttribute ( "reconcileflag", reconcileflag );
                split.setAttribute ( "memo", subop.getComment() );
                split.setAttribute ( "value", shape );
                split.setAttribute ( "account", date=="0000-00-00" ? "AStd::Equity" : ( cat.exist() ? cat.getUniqueID() : "" ) );
            }
        }

        opTreated.insert(iOperation.getUniqueID());
    }
    return err;
}

#include "skgimportkmy.moc"
